@@ -6,7 +6,7 @@
 #include <NTL/vector.h>
 #include <NTL/GF2E.h>
 #include <NTL/vec_GF2E.h>
-#include <NTL/FFT.h>
+#include <NTL/vec_long.h>
 #include <NTL/GF2XVec.h>
 #include <NTL/Lazy.h>
 
@@ -40,13 +40,8 @@ explicit GF2EX(const GF2& a) { *this = a; }
 
 GF2EX(INIT_SIZE_TYPE, long n) { rep.SetMaxLength(n); }
 
-GF2EX(const GF2EX& a) : rep(a.rep) { }
-
-
-GF2EX& operator=(const GF2EX& a) 
-   { rep = a.rep; return *this; }
-
-~GF2EX() { }
+// default copy constructor and assignment
+// default destructor
 
 void normalize();
 // strip leading zeros
@@ -100,6 +95,10 @@ void swap(GF2EX& x) { rep.swap(x.rep); }
 
 
 
+NTL_DECLARE_RELOCATABLE((GF2EX*))
+
+
+
 
 /********************************************************************
 
@@ -663,7 +662,6 @@ long InvModStatus(GF2EX& x, const GF2EX& a, const GF2EX& f);
 class GF2EXModulus {
 public:
    GF2EXModulus();
-   ~GF2EXModulus() { }
 
    GF2EXModulus(const GF2EX& ff);
 
@@ -686,6 +684,7 @@ public:
 }; 
 
 
+NTL_DECLARE_RELOCATABLE((GF2EXModulus*))
 inline long deg(const GF2EXModulus& F) { return F.n; }
 
 void build(GF2EXModulus& F, const GF2EX& f);
@@ -849,8 +848,9 @@ struct GF2EXArgument {
    vec_GF2EX H;
 };
 
-NTL_THREAD_LOCAL
-extern long GF2EXArgBound;
+extern 
+NTL_CHEAP_THREAD_LOCAL
+long GF2EXArgBound;
 
 
 void build(GF2EXArgument& H, const GF2EX& h, const GF2EXModulus& F, long m);
