@@ -36,10 +36,14 @@ inline ZZ_pE FindRoot(const ZZ_pEX& f)
 // assumes that f is monic and splits into distinct linear factors
 
 
-NTL_THREAD_LOCAL extern long ZZ_pEX_GCDTableSize; /* = 4 */
+extern 
+NTL_CHEAP_THREAD_LOCAL 
+long ZZ_pEX_GCDTableSize; /* = 4 */
 // Controls GCD blocking for NewDDF
 
-NTL_THREAD_LOCAL extern double ZZ_pEXFileThresh; 
+extern 
+NTL_CHEAP_THREAD_LOCAL 
+double ZZ_pEXFileThresh; 
 // of these tables exceeds ZZ_pEXFileThresh KB.
 
 
