@@ -6,11 +6,7 @@
 
 NTL_OPEN_NNS
 
-NTL_vector_decl(vec_long,vec_vec_long)
-
-NTL_eq_vector_decl(vec_long,vec_vec_long)
-
-NTL_io_vector_decl(vec_long,vec_vec_long)
+typedef Vec< Vec<long> > vec_vec_long;
 
 NTL_CLOSE_NNS
 
